@@ -86,12 +86,24 @@ int Internal::likely_phase (int idx) { return decide_phase (idx, false); }
 
 /*------------------------------------------------------------------------*/
 
+// adds new level to control and trail
+//
+void Internal::new_trail_level (int lit) {
+  level++;
+  control.push_back (Level (lit, trail.size ()));
+}
+
+/*------------------------------------------------------------------------*/
+
 bool Internal::satisfied () {
-  size_t assigned = trail.size ();
-  if (propagated < assigned)
-    return false;
   if ((size_t) level < assumptions.size () + (!!constraint.size ()))
     return false;
+  if (num_assigned < (size_t) max_var)
+    return false;
+  assert (num_assigned == (size_t) max_var);
+  if (propagated < trail.size ())
+    return false;
+  size_t assigned = num_assigned;
   return (assigned == (size_t) max_var);
 }
 
@@ -120,8 +132,7 @@ int Internal::decide () {
       res = 20;
     } else if (tmp > 0) {
       LOG ("assumption %d already satisfied", lit);
-      level++;
-      control.push_back (Level (0, trail.size ()));
+      new_trail_level (0);
       LOG ("added pseudo decision level");
       notify_decision ();
     } else {
@@ -176,8 +187,7 @@ int Internal::decide () {
            "is implied by assumptions",
            satisfied_lit);
 
-      level++;
-      control.push_back (Level (0, trail.size ()));
+      new_trail_level (0);
       LOG ("added pseudo decision level for constraint");
       notify_decision ();
 
